@@ -1,5 +1,5 @@
-init1.1.jk.j <-
-function(reference,response,L,K,t1,model,m1,mnr){
+init1.2.jk.j <-
+function(reference,response,L,K,m1,m2,t1,t2,model,mnr){
 x = reference
 y = response
 maxnr = mnr
@@ -25,8 +25,7 @@ if (floor(L[j]) != L[j]) stop("L should contain integers > 1")
 if (K < 1) stop("K should be positive integer")
 
 
-
-m = t1
+m = t2
 thresh <- -744
 if (floor(m)!= m) stop("number of iterations should be positive integer")
 
@@ -64,48 +63,34 @@ gamma[j,l] <- log(sum(y[,index[j]-1+l])) - s
 }}
 
 
-#number of em runs
-ggg<-m1
+
 
 iter <- 1
 
 
 
 
-glmest<-array(data = NA, dim = c(qq,1+tau))
-
-for (j in 1:qq){
-glmest[j,]<-coefficients(glm(y[,j]~x,family = poisson()))
-}
-
-
-
 if (model == 1){
 for (iter in 1:m){
 
+print(c("Overall small EM : run ", iter))
 
-
-
-tt<- -10^9
-
-
-for(mpla in 1:20){
+#conds = L
+n.comp<-K
+n.init=t1
 for (j in 1:q){
-for(k in 1:K){
-bs<-0.2*runif(1)
-beta[iter,j,k,]<-numeric(tau)
-alpha[iter,j,k]<-0
-for(l in 1:L[j]){
-#beta[iter,j,k,] <- beta[iter,j,k,] + coefficients(glm(y[,index[j]-1+l]~x,family = poisson()))[2:(1+tau)] + bs*rnorm(tau)
-#alpha[iter,j,k] <- alpha[iter,j,k] + coefficients(glm(y[,index[j]-1+l]~x,family = poisson()))[1] + 5*bs*rnorm(1)};
-beta[iter,j,k,] <- beta[iter,j,k,] + glmest[index[j]-1+l,2:(1+tau)] + bs*rnorm(tau)
-alpha[iter,j,k] <- alpha[iter,j,k] + glmest[index[j]-1+l,1] + 3*bs*rnorm(1)};
-alpha[iter,j,k] <- alpha[iter,j,k]/L[j]
-beta[iter,j,k,]<-beta[iter,j,k,]/L[j];
-}}
+#print(c("1st small EM: initializion condition ",j) )
+respj<-y[,index[j]:(index[j]+L[j]-1)]
+if (L[j]==1)dim(respj)<-c(length(respj),1)
+print(c("Sub-Small EM for the initializion of condition ",j))
+condsj = c(L[j])
+start<-init1.1.jk.j(reference = x,response = respj,L = condsj,K = n.comp,t1 = n.init,model = 1,m1=m1,mnr = maxnr)
+alpha[iter,j,]<-start$alpha
+beta[iter,j,,]<-start$beta
+}
+psim[iter,]<- start$psim
+if (q>1){psim[iter,]<- rep(1/K,K)}
 
-psim[iter,]<- rep(1,K) #runif(K)
-psim[iter,] <- psim[iter,]/sum(psim[iter,])
 # compute the current means
 condmeans = vector("list",length = K)
 ar<-array(data = NA, dim =c(n,qq))
@@ -122,17 +107,7 @@ condmeans[[k]]<-ar
 }
 
 
-lll<-mylogLikePoisMix(y, condmeans, psim[iter,])
-if(lll[[1]]>tt){tt<-lll[[1]];
-good.alpha<-alpha[iter,,]
-good.beta<-beta[iter,,,]
-good.condmeans<-condmeans
-}
-}
 
-alpha[iter,,]<-good.alpha
-beta[iter,,,]<-good.beta
-condmeans<-good.condmeans
 
 
 ########################################### liges epanalipseis tou em ###################################################
@@ -148,8 +123,7 @@ yy[,j]<-yy[,j] + y[,index[j]-1+l]
 yy[,j]<-yy[,j]/L[j]
 }
 st<-numeric(tau + 1)
-
-for (gr in 1:ggg){
+for (gr in 1:m2){
 ############################################################################################################################
 #E-Step: Mean allocation vectors   #
 ############################################################################################################################
@@ -158,8 +132,8 @@ for (k in 1:K){
 z[,k] <- z[,k] + rowSums(dpois(y,condmeans[[k]],log = T))
 }
 
-if (K>1){
 
+if (K>1){
 v1<-which(apply(z,1,max)< thresh)
 v3<-1:n
 len<-length(v1)
@@ -173,7 +147,6 @@ v3<- -v1
 z[v3,]<-exp(z[v3,])
 z<-z/rowSums(z)
 
-
 epsilon <- 1e-10
 sl<-length(z[z < epsilon])
 bl<-length(z[z > 1-epsilon])
@@ -189,6 +162,7 @@ for(j in 1:q){
 #run<-glm(yy[,j]~x,family=poisson, start = st, weights = z[,k])
 
 nrthreshold <- log(10^(-10));
+#maxnr = 3
 sc <- nrthreshold + 1
 theta[1] <- alpha[iter,j,k]# alpha[j,]'s
 theta[2:(tau+1)] <- beta[iter, j,k,]# beta_{j}
@@ -237,7 +211,7 @@ if (is.infinite(sc)==T)
 {theta<-theta1;sc<-35} else
 theta <- theta - qr.coef(qr(hessian,tol = 1e-300),grad)
 #print(theta)
-if (is.na(max(theta))==T) theta<-theta1 #theta<-runif(1+tau) 
+if (is.na(max(theta))==T){theta<-theta1} #else theta<-runif(1+tau)
 #if (sc>gr){theta<-runif(1+tau);print("oops")} else {
 #theta <- theta - qr.coef(qr(hessian,tol = 1e-300),grad)
 #}
@@ -272,8 +246,7 @@ condmeans[[k]]<-ar
 }
 
 
-#lll<-mylogLikePoisMix(y, condmeans, psim[iter,])
-#print(c("iteration ll = ",lll[[1]]))
+
 
 }
 
@@ -287,17 +260,17 @@ lll<-mylogLikePoisMix(y, condmeans, psim[iter,])
 
 
 ll[iter]<-lll[[1]]
-print(lll[[1]])
+print(paste("loglikelihood: ",lll[[1]]))
 
 
 }
 
-#print("end of initialization")
+print("end of initialization")
 
 ss<- 1:m
 mm<-max(ll)
 max.index<- ss[ll==mm]
-max.index<-max.index[[1]]
+max.index<-max.index[1]
 results<-list(psim[max.index,],alpha[max.index,,], beta[max.index,,,],ll[max.index])
 names(results)<-c("psim","alpha,","beta","ll")
 
@@ -312,40 +285,33 @@ if (model == 2){
 
 delta <- array(data = 0,dim = c(m,q))
 
-
-
-
 for (iter in 1:m){
 
 
+print(c("Overall small EM : run ", iter))
 
 
 
 
-
-
-tt<- -10^9
-
-
-for(mpla in 1:20){
-##############
-
-bs<-0.2*runif(1)
-alpha[iter,,] <- array(data = 20*bs*runif(q*K),dim = c(q,K)) - 40*bs
-for (j in 1:q) alpha[iter,j,] <- alpha[iter,j,] - mean(alpha[iter,j,])
-
-for (j in 1:q){ 
-delta[iter,j]<- 0;
-for (l in 1:L[j]){
-delta[iter,j] <- delta[iter,j] + glmest[index[j]-1+l,1];
-beta[iter,j,] <- beta[iter,j,] + glmest[index[j]-1+l,2:(1+tau)]
-}
-delta[iter,j]<- delta[iter,j]/L[j]
-beta[iter,j,]<- beta[iter,j,]/L[j]
+#############################
+#conds = L
+n.comp<-K
+n.init=t1
+for (j in 1:q){
+#print(c("initializing condition ",j) )
+respj<-y[,index[j]:(index[j]+L[j]-1)]
+if (L[j]==1)dim(respj)<-c(length(respj),1)
+print(c("Sub-Small EM for the initializion of condition ",j))
+condsj = c(L[j])
+start<-init1.1.jk.j(reference = x,response = respj,L = condsj,K = n.comp,t1 = n.init,model = 2,m1=m1,mnr = maxnr)
+alpha[iter,j,]<-start$alpha
+beta[iter,j,]<-start$beta
+delta[iter,j]<-start$delta
 }
 
-psim[iter,]<-rep(1/K,K)
-psim[iter,] <- psim[iter,]/sum(psim[iter,])
+psim[iter,]<- start$psim
+if (q>1){psim[iter,]<- rep(1/K,K)}
+
 
 
 # compute the current means
@@ -364,26 +330,6 @@ condmeans[[k]]<-ar
 }
 
 
-###############
-
-
-
-lll<-mylogLikePoisMix(y, condmeans, psim[iter,])
-if(lll[[1]]>tt){tt<-lll[[1]];
-good.alpha<-alpha[iter,,]
-good.beta<-beta[iter,,]
-good.condmeans<-condmeans
-good.delta<-delta[iter,]
-}
-}
-
-alpha[iter,,]<-good.alpha
-beta[iter,,]<-good.beta
-delta[iter,]<-good.delta
-condmeans<-good.condmeans
-
-
-
 
 
 
@@ -391,7 +337,7 @@ condmeans<-good.condmeans
 ###############################################################################################################################
 
 nrthreshold<- -6*log(10)
-for (gr in 1:ggg){
+for (gr in 1:m2){
 ############################################################################################################################
 #E-Step: Mean allocation vectors   #
 ############################################################################################################################
@@ -412,14 +358,14 @@ z[v1,]<- 0
 z[ddd]<- 1
 v3<- -v1 
 }
-z[v3,]<-exp(z[v3,])
+z[v3,]<-exp(z[v3,])}
 z<-z/rowSums(z)
 
 epsilon <- 1e-10
 sl<-length(z[z < epsilon])
 bl<-length(z[z > 1-epsilon])
 z[z<epsilon]<-rep(epsilon,sl)
-z[z>1-epsilon]<-rep(1-epsilon,bl)}
+z[z>1-epsilon]<-rep(1-epsilon,bl)
 z<-z/rowSums(z)
 
 
@@ -433,7 +379,7 @@ theta[(K+3):(K+2+tau)] <- beta[iter, j,]# beta_{j}
 theta1<-theta
 #iter1<-0
 # Main Loop of N-R iterations
-#maxnr=4
+#maxnr=10
 metritis = 1
 while (sc > nrthreshold&metritis<maxnr ) {
 metritis = metritis + 1
@@ -531,14 +477,14 @@ lll<-mylogLikePoisMix(y, condmeans, psim[iter,])
 
 
 ll[iter]<-lll[[1]]
-print(lll[[1]])
+print(paste("loglikelihood: ", lll[[1]]))
 }
 print("end of initialization")
 ss<- 1:m
 mm<-max(ll)
 max.index<- ss[ll==mm]
 max.index<-max.index[1]
-#print(c("iter selected = ", max.index))
+print(c("iter selected = ", max.index))
 results<-list(psim[max.index,],alpha[max.index,,], beta[max.index,,],delta[max.index,], ll[max.index])
 names(results)<-c("psim","alpha,","beta","delta", "ll")}
 return(results)}
